/*----------------------------------------------------------------
	author: massimo.sala.71@gmail.com
	license: MIT
	pls report bugs/improvements to me
*/

#include "config.h"

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#define safe_strlen(sz)		(sz == NULL ? 0 : strlen(sz))

#include "protocol.h"

// libmonit
#include "exceptions/IOException.h"
#include "exceptions/ProtocolException.h"


/**
 *  MS-SQL test. Default port: 1434/udp
 *  Optional argument: instance
 *
 *  @file
 */

#define SSRP_BCAST				0x02
#define SSRP_UNICAST			0x03
#define SSRP_INSTANCE			0x04
#define SSRP_SERVER_RESPONSE	0x05

#define INSTANCE_MAXLEN			16
#define REQUEST_MAXLEN			(1 + INSTANCE_MAXLEN + 1)

#define REPLY_MINLEN			(3 + 10+3 + 12+3 + 1)		// SSRP_SERVER_RESPONSE b1 b2 [ServerName;s1;InstanceName;s2;...;;]+
#define REPLY_MAXLEN			512


void check_mssql(Socket_T socket)
{
	Port_T port;
	const char * szInstance	= NULL;
	char probe[REQUEST_MAXLEN];
	char reply[REPLY_MAXLEN + 1];
	int br = 0;
	char * ptr;

	ASSERT(socket);
	port = Socket_getPort(socket);
	if (port != NULL)
	{
		szInstance = port->parameters.mssql.instance;
		br = safe_strlen(szInstance);
		if (br > INSTANCE_MAXLEN)
			THROW(ProtocolException, "MSSQL: instance name too long: %s -- %s", szInstance, STRERROR);
	}
	if (br == 0)
	{
		probe[0] = SSRP_UNICAST;
		br = 1;
	}
	else
	{
		probe[0] = SSRP_INSTANCE;
		strcpy(probe + 1, szInstance);
		br += 2;
	}

	Socket_nodelay(socket);
	if (Socket_write(socket, probe, br) <= 0)
		THROW(IOException, "MSSQL: error sending request -- %s", STRERROR);

	br = Socket_read(socket, reply, REPLY_MAXLEN);
	if (br <= 0)
		THROW(IOException, "MSSQL: did not receive answer from server -- %s", STRERROR);
	if (br < REPLY_MINLEN)
		THROW(ProtocolException, "MSSQL: Received %d bytes from server, expected at least %d bytes", br, REPLY_MINLEN);

//memo: instances can overflow REPLY_MAXLEN, just look for the first one
	reply[br] = 0;
	ptr = reply + 3;
	if (reply[0] != SSRP_SERVER_RESPONSE || strncmp(ptr, "ServerName;", 11) != 0 || strstr(ptr, ";;") == NULL)
		THROW(ProtocolException, "MSSQL: unknown server response (code %d) -- %s", reply[0], STRERROR);
}
